# R FUNCTIONS TO TEST FOR ABSOLUTE CONTINUITY OF DISTRIBUTIONS.
#
# Written by Radford M. Neal, March 2001.


# FIND RATIOS OF DISTANCE TO CLOSEST AND SECOND-CLOSEST POINTS IN A SAMPLE.

dist.ratios <- function (d)
{ 
  d <- as.matrix(d)
  n <- nrow(d)
  p <- ncol(d)

  r <- rep(0,n)

  for (i in 1:n)
  { s <- t(d)-d[i,]
    q <- sort(apply(s^2,2,sum),partial=c(1,2,3))
    r[i] <- q[2]/q[3]
  }

  sort(sqrt(r)^p)
}


# FIND RATIOS OF DISTANCE TO CLOSEST AND SECOND-CLOSEST POINTS IN A SAMPLE.
# Special 1D version.

dist.ratios1 <- function (d)
{ 
  n <- length(d)
  r <- rep(0,n)

  for (i in 1:n)
  { q <- sort(abs(d-d[i]),partial=c(1,2,3))
    r[i] <- q[2]/q[3]
  }

  sort(r)
}


# PLOT CUMULATIVE DISTRIBUTION FUNCTIONS.  Plots one or two CDFs for 
# data that ranges over (0,1), along with a dotted line for the CDF of
# a uniform distribution.

plot.cdf <- function (d1, d2=NULL, xlab="", ...)
{
  plot(c(0,1),c(0,1),type="l",lty=3,ylab="",xlab=xlab,...)

  lines((1:length(d1))/length(d1),d1,lwd=1)

  if (!is.null(d2))
  { lines((1:length(d2))/length(d2),d2,lwd=1)
  }
}
